import type { HowlOptions } from "howler";
declare global {
    interface HTMLElement {
        /**
         * Plays the sound
         * only works if the sound is created using the `svelte-sound` action
         */
        play: () => void;
        /**
         * Stops the sound
         * only works if the sound is created using the `svelte-sound` action
         */
        stop: () => void;
    }
}
/**
 * The events that is used to play the sound
 */
type PlayEvent = keyof HTMLElementEventMap;
/**
 * The events that is used to stop the sound
 */
type StopEvent = keyof HTMLElementEventMap;
type Options = {
    events: [PlayEvent, StopEvent?];
} & HowlOptions;
/**
 * Main Sound class
 * used to create and manage sounds by this library
 */
declare class Sound {
    private node;
    private options;
    private howl;
    constructor(node: HTMLElement, options: Options);
    /**
     * creates the sound
     */
    private create;
    update(options: Options): void;
    destroy(): void;
    /**
     * plays the sound
     */
    private play;
    /**
     * stops the sound
     */
    private stop;
    /**
     * Removes the event listeners attached to the node
     */
    private removeEventListeners;
    /**
     * unloads the sound
     */
    private unload;
}
/**
 * Svelte Action to play sound effects
 */
export declare function sound(node: HTMLElement, options: Options): Sound;
/**
 * Svelte Sound Reusable Action generator
 */
export declare function useSound(src: string | string[], events: [PlayEvent, StopEvent?], options?: Omit<Options, "src" | "events">): (node: HTMLElement, overrideOptions?: Partial<Options>) => Sound;
export {};
