var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class Sound {
  constructor(node, options) {
    __publicField(this, "node");
    __publicField(this, "options");
    __publicField(this, "howl");
    this.node = node;
    this.options = options;
    this.create();
  }
  /**
   * creates the sound
   */
  async create() {
    const { Howl } = await import("./howler.core-560d0a86.js").then((n) => n.h);
    const { src, events, loop, volume } = this.options;
    const sound2 = new Howl({
      src,
      loop: loop || false,
      volume: volume || 1,
      ...this.options
    });
    this.howl = sound2;
    const [playEvent, stopEvent] = events;
    this.node.play = this.play.bind(this);
    this.node.stop = this.stop.bind(this);
    this.node.addEventListener(playEvent, this.play.bind(this));
    this.node.addEventListener(stopEvent, this.stop.bind(this));
  }
  update(options) {
    this.unload();
    this.options = options;
    this.create();
  }
  destroy() {
    this.stop();
    this.unload();
  }
  /**
   * plays the sound
   */
  play() {
    this.howl.play();
  }
  /**
   * stops the sound
   */
  stop() {
    this.howl.stop();
  }
  /**
   * Removes the event listeners attached to the node
   */
  removeEventListeners() {
    const { events } = this.options;
    const [playEvent, stopEvent] = events;
    this.node.removeEventListener(playEvent, this.play.bind(this));
    this.node.removeEventListener(stopEvent, this.stop.bind(this));
  }
  /**
   * unloads the sound
   */
  unload() {
    this.removeEventListeners();
    this.howl.unload();
  }
}
function sound(node, options) {
  return new Sound(node, options);
}
function useSound(src, events, options) {
  return (node, overrideOptions) => new Sound(node, { src, events, ...options, ...overrideOptions });
}
export {
  sound,
  useSound
};
